#ifndef _FILE_SYSTEM_H
#define _FILE_SYSTEM_H

#ifndef M16C_62P
#include "datatypes.h"
#define far 
#endif
#ifdef M16C_62P
#include "inet/datatypes.h"
#endif

#define MAX_FILES 16
#define MAX_FILENAME 32
#define BLOCK_SIZE 32
#define MAX_BLOCK_COUNT 160
#define MAX_DIRFILES 8		/* max # files in each dir */

//This is the block structure used for tracking blocks of memory.
struct BLOCK
{
	UINT8 used;
	UINT8 block_data[BLOCK_SIZE];
	struct BLOCK * next_block;
};
typedef struct BLOCK block;

//This the the file table, for tracking file & directory information.
struct FILE_ENTRY
{
	INT8 type;						/* 0:unused  1:dir  2:file */
	char name[MAX_FILENAME+1];		/* NULL termintated string for filename */
	UINT16 size;					/* Total file size */
	INT8 num_dirfiles;				/* #files if it's a dir */
	INT8 dirfiles[MAX_DIRFILES];	/* list of fat_index of all files if it's a dir */
	block * start_block;			/* address of the first block */
};
typedef struct FILE_ENTRY file_entry;

INT8 FindFile(UINT32 name_str);
INT8 FindFilePath(UINT32 path, UINT32 name_str);
UINT16 GetFileSize( UINT8 fat_index );
UINT16 ReadFile(UINT8 fat_index, UINT16 offset, UINT8 * buffer, UINT16 bytes2read);
INT8 CreateFile(UINT32 path, UINT32 name_str);
INT8 DeleteFile(UINT8 fat_index);
UINT16 WriteFile(UINT8 fat_index, UINT16 offset, UINT8 * buffer, UINT16 bytes2write);
INT8 CreateDirectory(UINT32 path, UINT32 dirname);
INT8 MoveFile(UINT32 dir1, UINT32 file1, UINT32 dir2, UINT32 file2);
INT8 RemoveDirectory(UINT32 dirpath);
INT8 IsDirectory(UINT32 filepath);
INT8 Format();
INT8* List(UINT32 dirpath);
void SpaceStatus(UINT16 * used_files, UINT16 * used_blocks, UINT16 * used_space, UINT16 * bytes_total);
void DirListing(UINT8 dir = 0);
void DumpFile(UINT8 index, UINT8 * name);

#endif

