/************************************************/
/* fs_test.c                                    */
/* Mu-huan Chiang, Brandon Ellison              */
/*                                              */
/************************************************/

#include <stdio.h>
#include <string.h>
#include "file_system.h"
#include "mote.h"


/***------------------------------------------------------***/
/***  main()                                              ***/
/***------------------------------------------------------***/
int main( int argc, char **argv)
{
	UINT16 used_files, used_blocks, used_space, bytes_total;
	INT8 failed;
	INT8 fat_index;
	UINT16 size;
	UINT16 num_bytes; 
	INT8* list;
	INT8 k, iterations, iterations2;

	//UINT8 string_buffer[20] = {'a','b','c','d','e','f','g','h','i','j','k','l',,'m','n','o','p','q','r','s','t'};
	UINT8 string_buffer[20] = "abcdefghijklmnopqrs";
	UINT8 buffer[500] = {'\0',};
	UINT8 buffer2[500] = {'\0',};
	UINT8 file_name[10] = "file1.dat";
	
	/* start in a clean state */
	failed = 0;
	iterations = 3;
	iterations2 = 15;

	for (k = 0; k < iterations; k++)
	{
		DirListing();
		SpaceStatus(&used_files, &used_blocks, &used_space, &bytes_total);

		fat_index = Format();
		printf("Format  --> return %d\n", fat_index);

		DirListing();
		SpaceStatus(&used_files, &used_blocks, &used_space, &bytes_total);

		/* create file in the root directory, then access that file */	
		fat_index = CreateFile((UINT32)"/", (UINT32)file_name);
		printf("CreateFile(\"/file1.dat\")  --> return %d\n",fat_index);
		if (fat_index != 1) { printf("Failed!\n"); failed = 1; }
		
		num_bytes = WriteFile(fat_index, 0, string_buffer, 10);
		printf("WriteFile  --> return %d\n",num_bytes);
		if (num_bytes != 10) { printf("Failed!\n"); failed = 1; }

		num_bytes = WriteFile(fat_index, 10, string_buffer, 10);
		printf("WriteFile  --> return %d\n",num_bytes);
		if (num_bytes != 10) { printf("Failed!\n"); failed = 1; }

		num_bytes = WriteFile(fat_index, 20, string_buffer, 10);
		printf("WriteFile  --> return %d\n",num_bytes);
		if (num_bytes != 10) { printf("Failed!\n"); failed = 1; }

		num_bytes = WriteFile(fat_index, 30, string_buffer, 10);
		printf("WriteFile  --> return %d\n",num_bytes);
		if (num_bytes != 10) { printf("Failed!\n"); failed = 1; }

		size = GetFileSize(fat_index);
		printf("GetFileSize  --> return %d\n",size);
		if (size != 40) { printf("Failed!\n"); failed = 1; }

		num_bytes = ReadFile(fat_index, 0, buffer, GetFileSize(fat_index));
		printf("ReadFile  --> return %d\n",num_bytes);
		if (num_bytes != 40) { printf("Failed!\n"); failed = 1; }

		printf("\t ==>  %s\n",buffer);
		if (strstr((const char *)buffer, "abcdefghijabcdefghijabcdefghijabcdefghij") != (char *) buffer)
			{ printf("Failed!\n"); failed = 1; }

		/* test directory hierarchy */
		fat_index = CreateDirectory((UINT32)"/", (UINT32)"dir1");
		printf("CreateDirectory(\"/dir1\")  --> return %d\n",fat_index);
		if (fat_index != 2) { printf("Failed!\n"); failed = 1; }

		fat_index = CreateFile((UINT32)"/dir1", (UINT32)"file1.dat");
		printf("CreateFile(\"/dir1/file1.dat\")  --> return %d\n",fat_index);
		if (fat_index != 3) { printf("Failed!\n"); failed = 1; }

		list = List((UINT32)"/dir1");
		printf("List(/dir1)    -->  %d %d %d %d %d\n",list[0],list[1],list[2],list[3],list[4]);
		if (list[0] != 3) { printf("Failed!\n"); failed = 1; }

		fat_index = CreateFile((UINT32)"/dir1", (UINT32)"file2.dat");
		printf("CreateFile(\"/dir1/file2.dat\")  --> return %d\n", fat_index);
		if (fat_index != 4) { printf("Failed!\n"); failed = 1; }

		list = List((UINT32)"/dir1");
		printf("List(/dir1)    -->  %d %d %d %d %d\n",list[0],list[1],list[2],list[3],list[4]);
		if (list[1] != 4) { printf("Failed!\n"); failed = 1; }

		DirListing();
		SpaceStatus(&used_files, &used_blocks, &used_space, &bytes_total);

		printf("RemoveDirectory(/dir1)   --> return %d\n",RemoveDirectory((UINT32)"/dir1"));

		num_bytes = WriteFile(fat_index, 0, string_buffer, 10);
		printf("WriteFile  --> return %d\n",num_bytes);
		if (num_bytes != 10) { printf("Failed!\n"); failed = 1; }

		num_bytes = WriteFile(fat_index, 10, string_buffer, 10);
		printf("WriteFile  --> return %d\n",num_bytes);
		if (num_bytes != 10) { printf("Failed!\n"); failed = 1; }

		size = GetFileSize(fat_index);
		printf("GetFileSize  --> return %d\n",size);
		if (size != 20) { printf("Failed!\n"); failed = 1; }

		num_bytes = ReadFile(fat_index, 0, buffer2, GetFileSize(fat_index));
		printf("ReadFile  --> return %d\n",num_bytes);
		if (num_bytes != 20) { printf("Failed!\n"); failed = 1; }

		printf("\t ==>  %s\n",buffer2);
		if (strstr((const char *)buffer, "abcdefghijabcdefghij") != (char *) buffer)
			{ printf("Failed!\n"); failed = 1; }

		SpaceStatus(&used_files, &used_blocks, &used_space, &bytes_total);

		fat_index = FindFilePath((UINT32)"/", (UINT32)file_name);
		printf("FindFilePath(\"/file1.dat\")  --> return %d\n",fat_index);
		if (fat_index != 1) { printf("Failed!\n"); failed = 1; }

		//fat_index = DeleteFile(fat_index);
		//printf("DeleteFile(/file1.dat)  --> return %d\n",fat_index);

		fat_index = FindFilePath((UINT32)"/dir1", (UINT32)"file2.dat");
		fat_index = DeleteFile(fat_index);
		printf("DeleteFile(dir1/file2.dat)  --> return %d\n", fat_index);
		if (fat_index != 4) { printf("Failed!\n"); failed = 1; }

		list = List((UINT32)"/");
		printf("List(/)    -->  %d %d %d %d %d\n",list[0],list[1],list[2],list[3],list[4]);
		if (list[0] != 1) { printf("Failed!\n"); failed = 1; }
		if (list[1] != 2) { printf("Failed!\n"); failed = 1; }

		fat_index = CreateFile((UINT32)"/",(UINT32)"file3.dat");
		printf("CreateFile(\"/file3.dat\")  --> return %d\n", fat_index);
		if (fat_index != 4) { printf("Failed!\n"); failed = 1; }

		list = List((UINT32)"/dir1");
		printf("List(/dir1)    -->  %d %d %d %d %d\n",list[0],list[1],list[2],list[3],list[4]);
		if (list[0] != 3) { printf("Failed!\n"); failed = 1; }
		if (list[1] != -1) { printf("Failed!\n"); failed = 1; }

		list = List((UINT32)"/");
		printf("List(/)    -->  %d %d %d %d %d\n",list[0],list[1],list[2],list[3],list[4]);
		if (list[0] != 1) { printf("Failed!\n"); failed = 1; }
		if (list[1] != 2) { printf("Failed!\n"); failed = 1; }
		if (list[2] != 4) { printf("Failed!\n"); failed = 1; }
		
		fat_index = FindFilePath((UINT32)"/", (UINT32)"file1.dat");
		fat_index = DeleteFile(fat_index);
		printf("DeleteFile(/file1.dat)  --> return %d\n",fat_index);
		if (fat_index != 1) { printf("Failed!\n"); failed = 1; }

		list = List((UINT32)"/");
		printf("List(/)    -->  %d %d %d %d %d\n",list[0],list[1],list[2],list[3],list[4]);
		if (list[0] != 2) { printf("Failed!\n"); failed = 1; }
		if (list[1] != 4) { printf("Failed!\n"); failed = 1; }

		/* MoveFile ********************/
		DirListing();

		fat_index = MoveFile((UINT32)"/",(UINT32)"file1.dat",(UINT32)"/",(UINT32)"file2.dat");
		printf("MoveFile(  /file1.dat --> /file2.dat \t return %d\n",fat_index);
		if (fat_index != -1) { printf("Failed!\n"); failed = 1; }

		fat_index = MoveFile((UINT32)"/",(UINT32)"file3.dat",(UINT32)"/dir1",(UINT32)"file4.dat");
		printf("MoveFile(  /file3.dat --> /dir1/file4.dat \t return %d\n",fat_index);
		if (fat_index != 4) { printf("Failed!\n"); failed = 1; }

		DirListing();

		fat_index = FindFilePath((UINT32)"/",(UINT32)"file1.dat");
		printf("FindFilePath(\"/file1.dat\")  --> return %d\n",fat_index);
		if (fat_index != -1) { printf("Failed!\n"); failed = 1; }

		fat_index = FindFilePath((UINT32)"/",(UINT32)"file2.dat");
		printf("FindFilePath(\"/file2.dat\")  --> return %d\n",fat_index);
		if (fat_index != -1) { printf("Failed!\n"); failed = 1; }

		/* IsDirectory *******************/

		fat_index = IsDirectory((UINT32)"/dir1");
		printf("IsDirectory(/dir1)  --> return %d\n", fat_index);
		if (fat_index != 1) { printf("Failed!\n"); failed = 1; }

		fat_index = IsDirectory((UINT32)"/file2");
		printf("IsDirectory(/file2)  --> return %d\n", fat_index);
		if (fat_index != 0) { printf("Failed!\n"); failed = 1; }

		/* RemoveDirectory ****************/
		fat_index = RemoveDirectory((UINT32)"/dir1");
		printf("RemoveDirectory(/dir1)   --> return %d\n",fat_index);
		if (fat_index != -1) { printf("Failed!\n"); failed = 1; }

		fat_index = FindFilePath((UINT32)"/dir1", (UINT32)"file1.dat");
		fat_index = DeleteFile(fat_index);
		printf("DeleteFile(/dir1/file1.dat)  --> return %d\n",fat_index);
		if (fat_index != 3) { printf("Failed!\n"); failed = 1; }

		fat_index = FindFilePath((UINT32)"/dir1", (UINT32)"file4.dat");
		fat_index = DeleteFile(fat_index);
		printf("DeleteFile(/dir1/file4.dat)  --> return %d\n",fat_index);
		if (fat_index != 4) { printf("Failed!\n"); failed = 1; }

		fat_index = RemoveDirectory((UINT32)"/dir1");
		printf("RemoveDirectory(/dir1)   --> return %d\n",fat_index);
		if (fat_index != 2) { printf("Failed!\n"); failed = 1; }

		DirListing();
		SpaceStatus(&used_files, &used_blocks, &used_space, &bytes_total);

		/* Format ************************/
		Format();
		fat_index = FindFilePath((UINT32)"/",(UINT32)"file2.dat");
		printf("FindFilePath(\"/file2.dat\")  --> return %d\n",fat_index);
	}

	if (failed)
		printf("A test failed, check the listings.\n");
	else
		printf("All tests passed.\n");

	printf("\n");

	//Here is the test for the mote support:
	Format();

	for (k = 0; k < iterations2; k++)
	{
		fat_index = CreateFiles();
		printf("CreateFiles()  --> return %d\n", fat_index);
		if (fat_index != 0) { printf("Failed!\n"); failed = 1; }

		DirListing();
		SpaceStatus(&used_files, &used_blocks, &used_space, &bytes_total);
		
		fat_index = MoteUpdateTest( 1 );
		printf("MoteUpdateTest()  --> return %d\n", fat_index);

		DirListing();
		SpaceStatus(&used_files, &used_blocks, &used_space, &bytes_total);

		fat_index = UpdateHTMLIndex( 1 );
		printf("UpdateHTMLIndex()  --> return %d\n", fat_index);

		DirListing();
		SpaceStatus(&used_files, &used_blocks, &used_space, &bytes_total);

		fat_index = FindFilePath((UINT32)"/", (UINT32)"index.html");
		DumpFile(fat_index, (UINT8 *) "index_test.html");

		fat_index = FindFilePath((UINT32)"/", (UINT32)"back.jpg");
		DumpFile(fat_index, (UINT8 *) "back_test.jpg");
	}
}
