/************************************************/
/* Mote.c                                       */
/* Brandon Ellison, Mu-huan Chiang              */
/*                                              */
/************************************************/

#ifndef M16C_62P
#include <string.h>
#endif

#include "mote.h"
#include "file_system.h"
#include "template_html.h"
#include "back_jpg.h"
#include "http404_html.h"

void our_strcpy(far char * dest, far char * src);

//Internal Function
INT8 WriteMoteParameter(UINT8 file_index, UINT16 file_offset, UINT8 idx, mote_data * mdata);


/* InitZigbee                                        */
/* This function initialzes the wireless interface   */
/* of the Renesas board.                             */
/* Returns 0 on success, other on failure.           */

INT8 InitZigbee()
{
     /* Do initialization work here */
     return 0;
}

/* StopZigbee                                        */
/* This function shuts down the wireless interface   */
/* of the Renesas board.                             */
/* Returns 0 on success, other on failure.           */

INT8 StopZigbee()
{
     /* Do shut down work here */
     return 0;
}

/* QueryMote                                         */
/* This function sends a Mote device a request for   */
/* sensor data.  The Mote should respond shortly     */
/* with data or on its own schedule.  All you can do */
/* is ask nicely.                                    */
/* Returns 0 on success, other on failure.           */

INT8 QueryMote( UINT8 mote )
{
     /* Do query work here... Transmit a request packet. */
     return 0;
}

/* MotePacketIn                                      */
/* This function receives a data packet from a Mote  */
/* and places the incoming sensor data into a data   */
/* file.  If the Mote is sending something other     */
/* than sensor information, this will have to        */
/* dispatch the appropriate events.                  */
/* Returns 0 on success, other on failure.           */

INT8 MotePacketIn( void * packet_data )
{
     /* Do packet receive work here, parse the data. */
     return 0;
}

/* MoteUpdateTest                                    */
/* This function pretends that a Mote has generated  */
/* valid sensor data.  It creates the data and then  */
/* drops it into a file structure.  Then when the    */
/* http daemon wants to use it, the file is merged   */
/* with a template file to create a readable webpage.*/
/* Returns 0 on success, other on failure.           */

INT8 MoteUpdateTest( UINT8 mote )
{
	static INT8 i;
	INT8 fat_index;
	UINT16 num_bytes;
	UINT32 file_name[10];

    /* Create Mote data structure. */
    mote_data mdata;

     /* Fill the data with somewhat useful info. */
	if ((i < 0) || (i > 100))
		i = 5;
	mdata.humidity = 450 + i;
	mdata.exttemperature = 230 + i;
	mdata.light = 50 + (i/3);
	mdata.analog[0] = 30 + (i/10);
	mdata.analog[1] = 50 + (i/40);
	mdata.analog[2] = 0;
	mdata.analog[3] = 12 + (i/15);
	mdata.analog[4] = 50 + (i/12);
	mdata.analog[5] = 0;
	mdata.analog[6] = 0;
	mdata.analog[7] = 120 + (i/10);
	mdata.intvoltage = 25 + (i/20);
	mdata.inttemperature = 18 + (i/23);
    i++;

	//Create a filename.
	our_strcpy((far char *) file_name, (far char *) "motexx.dat");
	((unsigned char *) file_name)[4] = ((mote / 10) + '0');
	((unsigned char *) file_name)[5] = ((mote % 10) + '0');

	//Delete any previous mote file.
	fat_index = FindFilePath((UINT32)(far char *)"/", (UINT32)(far char *)file_name);
	if (fat_index != -1)
		fat_index = DeleteFile(fat_index);

	/* Write the data to a file */
	fat_index = CreateFile((UINT32)(far char *)"/", (UINT32)(far char *)file_name);
	if (fat_index)
	{
		num_bytes = WriteFile(fat_index, 0, (UINT8 *)&mdata, sizeof(mote_data));
		if (num_bytes != sizeof(mote_data))
			return 2;
	}     
    return 0;
}

/* UpdateHTMLIndex                                   */
/* This function upates the index files needed for   */
/* the http daemon.  This takes two files, the input */
/* template file and the mote data file and merges   */
/* them into an output file.                         */
/* Returns 0 on success, other on failure.           */

INT8 UpdateHTMLIndex( UINT8 mote )
{
	/* Create Mote data structure. */
	/* Read in the data from the Mote file */
	mote_data mdata;
	INT8 fat_index, fat_index2;
	UINT16 num_bytes;
	UINT32 file_name[10];
	UINT16 bytes_read, bytes_total, bytes_write;
	UINT8 buffer[5];

	//Create a filename.
	our_strcpy((far char *) file_name, (far char *)"motexx.dat");
	((unsigned char *) file_name)[4] = ((mote / 10) + '0');
	((unsigned char *) file_name)[5] = ((mote % 10) + '0');

	//Read the old mote file.
	fat_index = FindFilePath((UINT32) (far char *) "/", (UINT32) (far char *)file_name);
	if (fat_index == -1)
		return 2;

	num_bytes = ReadFile(fat_index, 0, (UINT8 *)&mdata, sizeof(mote_data));
	if (num_bytes != sizeof(mote_data))
		return 3;

	fat_index = FindFilePath((UINT32)(far char *)"/", (UINT32)(far char *)"template.html");
	if (fat_index == -1)
		return 4;

	/* Create the template file */
	fat_index2 = FindFilePath((UINT32)(far char *)"/", (UINT32)(far char *)"index.html");
	if (fat_index2 != -1)
		fat_index2 = DeleteFile(fat_index2);
	fat_index2 = CreateFile((UINT32)(far char *)"/", (UINT32)(far char *)"index.html");

	bytes_total = GetFileSize(fat_index);
	bytes_write = 0;
	bytes_read = 0;
	while (bytes_read < (bytes_total - 3))
	{
		num_bytes = ReadFile(fat_index, bytes_read, buffer, 4);
		if (num_bytes != 4)
			return 5;

		if ((buffer[2] == '#') && (buffer[1] == '#') && (buffer[0] == '#'))
		{
			bytes_write += WriteMoteParameter(fat_index2, bytes_write, buffer[3], &mdata);
			bytes_read += 4;
		}
		else if ((buffer[2] == '#') && (buffer[1] == '#') && (buffer[0] != '#'))
		{
			num_bytes = WriteFile(fat_index2, bytes_write, buffer, 1);
			if (num_bytes != 1)
				return 6;
			bytes_write += 1;
			bytes_read += 1;
		}
		else if ((buffer[2] == '#') && (buffer[1] != '#') && (buffer[0] != '#'))
		{
			num_bytes = WriteFile(fat_index2, bytes_write, buffer, 2);
			if (num_bytes != 2)
				return 7;
			bytes_write += 2;
			bytes_read += 2;
		}
		else if ((buffer[2] != '#'))
		{
			num_bytes = WriteFile(fat_index2, bytes_write, buffer, 3);
			if (num_bytes != 3)
				return 8;
			bytes_write += 3;
			bytes_read += 3;
		}
	}

	return 0;
}

/* CreateFiles                                       */
/* This function creates the files needed for all    */
/* of the functions of the run.  This includes a     */
/* template file and the image file.                 */
/* Returns 0 on success, other on failure.           */

INT8 CreateFiles()
{
	INT8 fat_index;
	UINT16 num_bytes;

	/* Create the Not Found file */
	// This is for the hardcoded https_callbacks stuff
	fat_index = FindFilePath((UINT32)(far char *)"/", (UINT32)(far char *)"http404.html");
	if (fat_index != -1)
		fat_index = DeleteFile(fat_index);
	fat_index = CreateFile((UINT32)(far char *)"/", (UINT32)(far char *)"http404.html");

	num_bytes = WriteFile(fat_index, 0, http404_html, http404_html_SIZE);
	if (num_bytes != http404_html_SIZE)
		return 3;

	/* Create the Index file */
	// This is for the hardcoded https_callbacks stuff
	fat_index = FindFilePath((UINT32)(far char *)"/", (UINT32)(far char *)"index.html");
	if (fat_index != -1)
		fat_index = DeleteFile(fat_index);
	fat_index = CreateFile((UINT32)(far char *)"/", (UINT32)(far char *)"index.html");

	num_bytes = WriteFile(fat_index, 0, http404_html, http404_html_SIZE);
	if (num_bytes != http404_html_SIZE)
		return 3;

	/* Create the template file */
	//Delete the old one if it exists
	fat_index = FindFilePath((UINT32) (far char *) "/", (UINT32) (far char *)"template.html");
	if (fat_index != -1)
		fat_index = DeleteFile(fat_index);
	fat_index = CreateFile((UINT32)(far char *)"/", (UINT32)(far char *)"template.html");

	num_bytes = WriteFile(fat_index, 0, template_html, template_html_SIZE);
	if (num_bytes != template_html_SIZE)
		return 1;

	/* Create the background file */
	fat_index = FindFilePath((UINT32)(far char *)"/", (UINT32)(far char *)"back.jpg");
	if (fat_index != -1)
		fat_index = DeleteFile(fat_index);
	fat_index = CreateFile((UINT32)(far char *)"/", (UINT32)(far char *)"back.jpg");

	num_bytes = WriteFile(fat_index, 0, back_jpg, back_jpg_SIZE);
	if (num_bytes != back_jpg_SIZE)
		return 2;

     
     return 0;
}

/* WriteMoteParameter                                */
/* This function writes a mote parameter out in      */
/* decimal format.  The parameter is chosen by the   */
/* character index.                                  */
/* Returns # of bytes on success, other on failure.  */

INT8 WriteMoteParameter(UINT8 file_index, UINT16 file_offset, UINT8 idx, mote_data * mdata)
{
	UINT8 buffer[7];
	UINT16 value;
	UINT8 offset;

	offset = 0;
	value = ((UINT16 *)mdata)[idx - 'A'];
	if (value > 1000)
	{
		buffer[offset] = (value / 1000) + '0';
		offset++;
		value = value - ((value / 1000) * 1000);
	}
	if (value > 100)
	{
		buffer[offset] = (value / 100) + '0';
		offset++;
		value = value - ((value / 100) * 100);
	}

	buffer[offset] = (value / 10) + '0';
	offset++;
	value = value - ((value / 10) * 10);

	buffer[offset] = '.';
	offset++;

	buffer[offset] = value + '0';
	offset++;

	return WriteFile(file_index, file_offset, buffer, offset);
}


