/************************************************/
/* Created by Renesas Technology America, Inc. */
/* Makers of M16C, H8, SH                       */
/* http://www.renesas.com                       */
/************************************************/

/* Tab spacing = 4 */
#include <inet\system.h>

#include <inet\http\http_server.h>
#include "file_system.h"
#include "mote.h"

unsigned char port4;

/* Notes:
    This demo has been created to be used on the following processors and boards
	M16C/62P      SKP16C62P board
                   - 8x2 LCD
				   - 3 buttons
				   - AD turn pot
                  SKPCOMMS board
				   - SMSC LAN96C111 Ethernet chip

    H8S/2674R     EDOSK2674R
                   - SMSC LAN96C96 Ethernet chip
                  Custom add-on board
                   - 8x2 LCD
				   - 4 buttons
				   - AD turn pot

    SH2-DSP/7616  MS7616SE01 Solution Engine board
	               - 8 LEDs
				   - 4 DIP switches
*/

/* Network Interface definition. Must be somewhere. */
struct netif localmachine;

/* Prototypes for functions in this file */
INT32 udp_eventlistener(INT8 cbhandle, UINT8 event, UINT32 ipaddr, UINT16 port, UINT16 buffindex, UINT16 datalen);
INT32 tcp_server_eventlistener(INT8 cbhandle, UINT8 event, UINT32 par1, UINT32 par2);
void PollButtons_isr(void);		/* Timer ISR */
void SendButtonInfo(void);
UINT8 SendStautsInfo(void);


/* TCP communications */
UINT8 tcp_in_arrived = 0;
UINT8 tcp_in_buffer[200];
extern INT8 tcps_demo_soch;		/* from tcp_server_demo.c */
UINT8 connection_established = 0;
UINT32 remote_ip;
UINT8 update_button_info = 0;

/* Global Variables */
INT8 udp_io_socket;			/* UDP Socket Handle */
INT8 tcp_server_socket;		/* TCP Socket Handle */
UINT8 buttons_pressed = 0;	/* Hold which buttons are currently pressed */
UINT32 total_seconds = 0;		/* Keep track of total seconds from bootup */

UINT8 lcd_cur_display = 0;	/* 0=Text, 1=Slider */
#define DISP_TEXT 0
#define DISP_SLIDER 1


/* The LCD driver will update these strings every a message is written */
extern char lcd_text1[],lcd_text2[];


/* Button port definistion */
#if H8S_2674
 /* H8S/2674 */
 #define BTN1_PIN P2.PORT.bit.B0
 #define BTN2_PIN P2.PORT.bit.B1
 #define BTN3_PIN P2.PORT.bit.B2
 #define BTN4_PIN P2.PORT.bit.B3
#endif

#if SH_7616
// S8 (pins 5,6,7,8)
 #define BTN1_PIN (*(unsigned char *)(0x2100003) & 0x01)
 #define BTN2_PIN (*(unsigned char *)(0x2100003) & 0x02)
 #define BTN3_PIN (*(unsigned char *)(0x2100003) & 0x04)
 #define BTN4_PIN (*(unsigned char *)(0x2100003) & 0x08)
#endif

#if M16C_62P
 /* SKP16C62P Board */
 #define BTN1_PIN p8_3
 #define BTN2_PIN p8_2
 #define BTN3_PIN p8_1
 #define BTN4_PIN p8_5
#endif

#if M32C_83
 /* SKP32C83 Board */
 #define BTN1_PIN p8_3
 #define BTN2_PIN p8_2
 #define BTN3_PIN p8_1
 #define BTN4_PIN p8_5
#endif

#if M32C_85
 /* SKP32C85 Board */
 #define BTN1_PIN p8_3
 #define BTN2_PIN p8_2
 #define BTN3_PIN p8_1
 #define BTN4_PIN p8_5
#endif



extern struct tcb tcp_socket[];

void main(void)
{
	INT16 len;
	UINT16 i;
	UINT8 ad_value = 0, send_ad_value = 0, stream_ad = 0;
	UINT8 send_status = 0;

	char buffer[] = {0,1,2,3,4,5,6,7,0};	// Using fancy 'R'
//	char buffer[] = {'R',2,3,4,5,6,7,0};	// Using standard 'R'

   	/* Program in static IP address */
	localmachine.localip = 	(UINT32)(172 * 0x1000000) + 
							(UINT32)(27 * 0x10000) +
							(UINT32)(52 * 0x100) +
							(UINT32) 45 ;					/* 172.27.52.45 */
/*	localmachine.localip = 	(UINT32)(10 * 0x1000000) + 
							(UINT32)(10 * 0x10000) +
							(UINT32)(10 * 0x100) +
							(UINT32) 20 ;					/* 10.10.10.20 */

    /* Default gateway */
	localmachine.defgw = 	(UINT32)(172 * 0x1000000) + 
							(UINT32)(27 * 0x10000) +
							(UINT32)(52 * 0x100) +
							(UINT32) 1 ;					/* 172.27.52.1 */
/*	localmachine.defgw = 	(UINT32)(10 * 0x1000000) + 
							(UINT32)(10 * 0x10000) +
							(UINT32)(10 * 0x100) +
							(UINT32) 1 ;					/* 10.10.10.1 */

   	/* Subnet mask */
   	localmachine.netmask = 0xFFFFFE00;						/* 255.255.254.0 */

   	/* Ethernet (MAC) address [5]:[4]:[3]:[2]:[1]:[0]*/
   	localmachine.localHW[5] = 0x08;
   	localmachine.localHW[4] = 0x00;
   	localmachine.localHW[3]	= 0x70;
   	localmachine.localHW[2]	= 0x22;
   	localmachine.localHW[1]	= 0x44;
   	localmachine.localHW[0]	= 0x55;

	/* Init system resources like timers, io ports, LCD, etc..*/
	init();
	timer_pool_init();

	DisplayString(LCD_LINE1,buffer);	// Display Renesas logo
	//Changed for CSC714 - Brandon
	DisplayString(LCD_LINE2,"CSC714");

	i = get_timer();
	init_timer(i,2*TIMERTIC);		/* Set timer to count down for 2 seconds */
	while( check_timer(i) );		/* Wait for timer to expire */
	free_timer( i );

	//Changed for CSC714 - Brandon
	//Format the file system
	Format();
	CreateFiles();
	//Changed for CSC714 - Brandon
	DisplayString(LCD_LINE2,"FileInit");

   	/* Initialize all network layers	*/
   	NE2000Init(&localmachine.localHW[0]);
   	arp_init();
   	udp_init();
   	tcp_init();
	https_init();

	/* Set the MTU of the http sockets to something higher than the default so we 
	/* can send back more data in a packet at a time */

	for(i=0; i < NO_OF_TCPSOCKETS; i++)
	{	
		if(tcp_socket[i].locport == HTTPS_SERVERPORT)
			tcp_socket[i].send_mtu = 1400 + MIN_TCP_HLEN; // this will allow 1400 bytes of HTTP at once
	}

	/* Transmit a packet for Debug */
//	arp_find (localmachine.defgw, &localmachine, ARP_TEMP_IP);

	/* Create socket for UDP streaming, then save returned handle */
	udp_io_socket = udp_getsocket(	0,
									udp_eventlistener,		// function to be called with UDP data arrives
									UDP_OPT_SEND_CS | UDP_OPT_CHECK_CS);
	if(udp_io_socket == -1)
	{
		DEBUGOUT("No free UDP sockets!! \r\n");
		while(1);	/* ERROR */
	}
	udp_open(udp_io_socket,5004);	// open that socket for receiving/sending data on port 5004

	/* Create socket for TCP communications */
	tcp_server_socket = tcp_getsocket(	TCP_TYPE_SERVER,	// type of TCP socket is server
									  	TCP_TOS_NORMAL,		// no other type of service implemented so far
									  	TCP_DEF_TOUT,		// timeout value in seconds.
									  	tcp_server_eventlistener);	// pointer to event listener function
	if( tcp_server_socket < 0 )
	{
		DEBUGOUT("TCP server unable to get socket!!\r\n");
		while(1);	/* ERROR */
	}
	tcp_listen(tcp_server_socket, 5001);	// Open that socket for receiving data on port 5001

	DisplayString(LCD_LINE1,"OpenTCP ");
	DisplayString(LCD_LINE2,"No Conn ");
	
	
	DEBUGOUT("< Entering to MAIN LOOP >\n\r");
   	while(1)
   	{
		/*********************************/
   		/* Try to receive Ethernet Frame */
		/*********************************/
   		if( NETWORK_CHECK_IF_RECEIVED() == TRUE )
   		{
   			switch( received_frame.protocol)
   			{
   				case PROTOCOL_ARP:
   					process_arp(&received_frame);	
	    			break;
   				case PROTOCOL_IP:   			
   					len = process_ip_in(&received_frame);
   					if(len < 0)
   						break;
   					switch(received_ip_packet.protocol)
					{
   						case IP_ICMP:
    						process_icmp_in (&received_ip_packet, len);    					
   							break;
   						case IP_UDP:
   							process_udp_in (&received_ip_packet,len);
   							break;
   						case IP_TCP:
    						process_tcp_in (&received_ip_packet, len);				
   							break;
   						default:
							break;
					}
   					break;
		   		default:
   					break;
   			}
			/* discard received frame */    		
    		NETWORK_RECEIVE_END();
    	}

		/*******************************************/
    	/* Check to see if TCP IN data has arrived */
		/*******************************************/
		if( tcp_in_arrived )
		{
			tcp_in_arrived = 0;		/* Reset flag */

			// Find out what the command is
			switch( tcp_in_buffer[0] )
			{
				case 1:		/* Print out what we received to LCD */
							/* Insert NULL to mark the end of the first line */
							/* Window GUI will always give us 16 char 8x2) */
							i = tcp_in_buffer[9];		// save 1st char of 2nd line
							tcp_in_buffer[9] = 0;		// LCD takes 8 chars per line
							DisplayString(LCD_LINE1,(char *)tcp_in_buffer+1);	// Display line 1
							tcp_in_buffer[9] = (char) i;	// Replace 2nd line char
							/* Insert NULL to make the end of the second line */
							tcp_in_buffer[17] = 0;		// LCD takes 8 chars per line
							DisplayString(LCD_LINE2,(char *)tcp_in_buffer+9);
							lcd_cur_display = DISP_TEXT;		/* signal LCD was used to write text */
							break;
				case 2:		/* Start streaming UDP values */
							stream_ad = 1;
							break;
				case 3:		/* Stop streaming UDP values */
							stream_ad = 0;
							break;
				case 4:		/* Send back current status */
							send_status = 1;
							break;
			}
		}


		/******************************************/
		/* Read AD to determine current pot value */
		/******************************************/
		/* If we are suppose to be streaming AD values from the pot,
		/* check to see if the ad value is the same as it was last time.
		/* Only send a new UDP packet with the AD value if the pot has
		/* been changed */
		#ifdef M16C_62P
		 #define AD_START_BIT adst
		 #define AD_RESULT ad0
		#endif
		#ifdef M32C_83
		 #define AD_START_BIT adst_ad0con0
		 #define AD_RESULT ad00
		#endif
		#ifdef M32C_85
		 #define AD_START_BIT adst_ad0con0
		 #define AD_RESULT ad00
		#endif
		#if H8S_2674
		 #define AD_START_BIT AD.ADCSR.bit.ADST
			// Since we only want 8-bit AD value, we need to drop the bottom 2 bits.
			// The 10-bit AD value is stored in bit 6-15 in the ADDRA
		 #define AD_RESULT (AD.ADDRA >> 8)
		#endif

		#if (SH_7616 != 1)		/* no AD on SH/7616 */
		if( stream_ad )
		{
			/* Check pot to see if the value has changed */
			AD_START_BIT = 1;			// Start the AD conversion
			while( AD_START_BIT );		// Wait till conversion is done
			if( ad_value != AD_RESULT )	// Compare new value with old one
			{
				ad_value = AD_RESULT;	// Save new value
				send_ad_value = 1;		// flag that new value should be sent
			}
		}
		#endif

		/***************************/
		/* Send UDP data if needed */
		/***************************/
		/* Check if we should send a new AD value. If an error occurs when we
		/* try to send the UDP packet, we'll try again later */
		if(send_ad_value)
		{
			/* Message needs to start from UDP_APP_OFFSET because TCP/IP stack will */
			/* put headers in front of the message to avoid data copying */
			net_buf[UDP_APP_OFFSET] = ad_value;

			/* send message */
			i = udp_send( udp_io_socket,
						  remote_ip,
						  5004,							// Remote port
						  net_buf+UDP_APP_OFFSET,		// Start of data
						  NETWORK_TX_BUFFER_SIZE-UDP_APP_OFFSET,
						  1);
			if( i > 0)
				send_ad_value = 0;
		}


		/*************************************************/
		/* Check button info needs to be updated via TCP */
		/*************************************************/
		if( update_button_info && connection_established )
		{
			SendButtonInfo();
		}

		/**********************************/
		/* Send back packet count via TCP */
		/**********************************/
		if( send_status && connection_established )
		{
			if( SendStautsInfo() )
				send_status = 0;	// Reset flag only if function was successful
									// If not, then try again the next itteration
		}

		//Changed for CSC714 - Brandon
		MoteUpdateTest( 1 );
		UpdateHTMLIndex( 1 );
		DisplayString(LCD_LINE2,"Update ");


    	/* TCP/IP stack Periodic tasks	*/
    	arp_manage();	// manage arp cache tables
    	tcp_poll();		// manage opened TCP connections (retransmissions, timeouts,...)

		https_run();	// needs to be called periodically
    }
}

/*
 * Event listener invoked when TCP/IP stack receives UDP datagram for
 * a given socket. Parameters:
 * - cbhandle 	- handle of the socket this packet is intended for. Check it
 *	              just to be sure, but in general case not needed
 * - event 		- event that is notified. For UDP, only UDP_EVENT_DATA
 * - ipaddr 	- IP address of remote host who sent the UDP datagram
 * - port 		- port number of remote host who sent the UDP datagram
 * - buffindex 	- buffer index in the receivebuffer allowing you to read 
 * 				  received data more than once from Ethernet controller by
 *				  invoking NETWORK_RECEIVE_INITIALIZE(buffindex) and then start
 *				  reading the bytes all over again
 */ 
INT32 udp_eventlistener(INT8 cbhandle, UINT8 event, UINT32 ipaddr, UINT16 port, UINT16 buffindex, UINT16 datalen)
{
	UINT16 i;
	UINT8 in_buffer[50];
	char text[5];
//	if(cbhandle!=udp_socket){
//		DEBUGOUT("Not my handle!!!!");
//		return (-1);
//	}
	
	switch(event){
		case UDP_EVENT_DATA:
			
			/* read data that was received (and 
			 * probably do something with it :-)
			 */
			if( lcd_cur_display != DISP_SLIDER )
			{
				/* So we only have to clear the LCD befor the first slider display */
				DisplayString(LCD_LINE1,"Slider  ");
				DisplayString(LCD_LINE2,"        ");
				lcd_cur_display = DISP_SLIDER;	/* signal LCD was used to write slider value */
			}

			for(i=0;i<datalen;i++)
				in_buffer[i] = RECEIVE_NETWORK_B();
				
			if( in_buffer[0] == 100 )
			{
				DisplayString(LCD_LINE2,"100%");
			}
			else
			{
				text[0] = ' ';
				if( in_buffer[0] > 9 )
					text[1] = (in_buffer[0] / 10) + '0';
				else
					text[1] = ' ';
				text[2] = (in_buffer[0] % 10) + '0';
				text[3] = '%';
				text[4] = 0;
				DisplayString(LCD_LINE2,text);
			}
			/* If needed initialize data sending
			 * by setting udp_demo_senddata variable or
			 * send data directly from event listener (
			 * only possible for UDP applications!!!!
			 */
			 
			break;
			
		default:
			/* should never get here */
			DEBUGOUT("Unknown UDP event :-(");
			break;
	}
	return 0;
}

 /*
 * Event listener invoked when TCP/IP stack receives TCP data for
 * a given socket. Parameters:
 * - cbhandle - handle of the socket this packet is intended for. Check it
 *	just to be sure, but in general case not needed
 * - event - event that is notified. For TCP there are quite a few possible
 *	events, check switch structure below for more information
 * - par1, par2 - parameters who's use depends on the event that is notified
 */
INT32 tcp_server_eventlistener(INT8 cbhandle, UINT8 event, UINT32 par1, UINT32 par2)
{
/* This function is called by TCP stack to inform about events	*/
	UINT16 i;
	
//	if( cbhandle != tcps_demo_soch)		/* Not our handle	*/
//		return(-1);
	
	switch( event )
	{
		/* Connection request event. Used by TCP/IP stack to inform
		 * the application someone is trying to establish a connection.
		 * Server can decide, based on provided IP address and port number,
		 * whether to allow or not connection establishment.
		 * Parameters:
		 *  - par1 - remote hosts IP address
		 *  - par2 - remote hosts port number
		 * 
		 * Return values from event listener:
		 * -1 - do not allow connection to be established (send reset)
		 * -2 - do not send any response for now to the SYN packet (let's
		 *		think a little before answering)
		 * 1  - allow connection to be established
		 */
		case TCP_EVENT_CONREQ:
			DEBUGOUT("Connection request arrived!\r\n");
			
			/* Enable all connections	*/
			return(1);
		
			break;
			
		/* Connection abort event. Connection on a given socket is beeing 
		 * aborted for somereason (usually retransmissions are used up or 
		 * some abnormal situation in communication happened).
 		 * Parameters:
		 *  - par1 - remote hosts IP address
		 *  - par2 - remote hosts port number
		 */
		case TCP_EVENT_ABORT:
			DEBUGOUT("Connection aborting!\r\n");
			break;
		
		/* Connection established event - three-way handshaking performed
		 * OK and connection is established.
		 * Parameters:
		 *  - par1 - remote hosts IP address
		 *  - par2 - remote hosts port number
		 */
		case TCP_EVENT_CONNECTED:
			DEBUGOUT("TCP connection established!\r\n");
			DisplayString(LCD_LINE2,"Conn Est");
			remote_ip = received_ip_packet.sip;
			connection_established = 1;
			break;
			
		/* Connection closing event. Happens when TCP connection is
		 * intentionally close by some side calling close function and
		 * initializing proper TCP connection close procedure.
		 * Parameters:
		 *  - par1 - remote hosts IP address
		 *  - par2 - remote hosts port number
		 */
		case TCP_EVENT_CLOSE:
			DEBUGOUT("TCP Connection closing...!\r\n");
			connection_established = 0;
			DisplayString(LCD_LINE1,"OpenTCP ");
			DisplayString(LCD_LINE2,"No Conn ");
			break;
			
		/* Data acknowledgment event. Happens when data that was
		 * previously sent gets acknowledged. This means we can now
		 * send some more data! :-)
		 * Parameters:
		 *  - par1 - remote hosts IP address
		 *  - par2 - remote hosts port number
		 */
		case TCP_EVENT_ACK:
			DEBUGOUT("Data acknowledged!\r\n");
			/* if more data should be sent, adjust variables and
				set tcps_demo_senddata variable */
				
			break;
		
		/* Data received event. Happens when we receive some data over the
		 * TCP connection.
		 * Parameters:
		 *  - par1 - number of data bytes received
		 *  - par2 = 0
		 */
		case TCP_EVENT_DATA:
			DEBUGOUT("Data arrived!\r\n");
			/* read data that was received (and 
			 * probably do something with it :-)
			 */
			for(i=0;i<par1;i++)
				tcp_in_buffer[i] = RECEIVE_NETWORK_B();
			tcp_in_arrived = par1;		/* Signal that new data has arrived */

			/* If needed initialize data sending
			 * by setting tcps_demo_senddata variable
			 */
			break;
			
		/* Regenerate data event. Happens when data needs to be
		 * retransmitted because of possible loss on the network.
		 * Note that THE SAME DATA must be sent over and over again
		 * until TCP_EVENT_ACK is generated (for that data)! 
		 * Parameters:
		 *  - par1 - amount of data to regenerate (usually all)
		 *	- par2 = 0
		 */
		case TCP_EVENT_REGENERATE:
			SendButtonInfo();
			// SEND AGAIN

			break;
	
		default:
			return(-1);
	}
	return(1);
}

extern char index_html[];

/* This isr will go off every 50 ms in order to check the status of the buttons */
#if (H8S_2674)
#pragma interrupt (PollButtons_isr)
#endif
#if (SH_7616)
#pragma interrupt (PollButtons_isr)
#endif

#if (M16C_62P)
#pragma INTERRUPT PollButtons_isr
#endif
#if (M32C_85)
#pragma INTERRUPT PollButtons_isr
#endif
#if (M32C_83)
#pragma INTERRUPT PollButtons_isr
#endif
void PollButtons_isr(void)
{
	/* Poll the buttons */
	UINT8 old_value;
	old_value = buttons_pressed;		// Save old value
	buttons_pressed	= 0;				// clear out all values
	if( !BTN1_PIN )
		buttons_pressed |= 1;			/* Button 1 is down */
	if( !BTN2_PIN )
		buttons_pressed |= 2;			/* Button 2 is down */
	if( !BTN3_PIN )
		buttons_pressed |= 4;			/* Button 3 is down */
	if( !BTN4_PIN )
		buttons_pressed |= 8;			/* Button 4 is down */

	/* Signal to send out a TCP packet if the button values have changed */
	if( buttons_pressed != old_value)
	{
		update_button_info = 1;
	}

	#ifdef H8S_2674
	TPU2.TSR.bit.TGFA = 0;	/* clear A compare/match bit */
	#endif
	#if SH_7616
		TPU.TSR2.bit.TGFA = 0;		/* clear A compare/match bit */
	#endif
}

/* we put this in a separate function so we can call it if we need to do a retransmit */
void SendButtonInfo(void)
{
	/* Put message in buffer. Message needs to start from TCP_APP_OFFSET
 	 * because TCP/IP stack will put headers in front of the message to
 	 * avoid data copying
 	 */
	/* First check if we can send data */
	if(tcp_checksend(tcp_server_socket) < 0 )
	{
		/* Not yet */
		/* Drop out and try again later */
	}
	else
	{
		net_buf[TCP_APP_OFFSET] = 1;					// Signal this will be a button press packet
		net_buf[TCP_APP_OFFSET+1] = buttons_pressed;	// put in current button press info
		/* send data */
		tcp_send(	tcp_server_socket,			// handle of TCP socket to be used for sending data
					&net_buf[TCP_APP_OFFSET],	// pointer to data buffer (start of user data)
					NETWORK_TX_BUFFER_SIZE - TCP_APP_OFFSET,	// buffer length in bytes (without space reserved at the beginning of buffer for headers)
					2);				// length of user data to be sent (in bytes)
		update_button_info = 0;		// Clear out button send flag
	}
}


struct _ETH_INFO
{
	UINT32 total_rx_packets;
	UINT32 total_tx_packets;
	UINT8 full_duplex:1;
	UINT8 speed_100:1;
	UINT8 linked:1;
};

// In Ethernet driver
extern struct _ETH_INFO eth_info;

/* we put this in a separate function so we can call it if we need to do a retransmit */
UINT8 SendStautsInfo(void)
{
	UINT32 tmp;
	UINT8 * pbuf;

	/* Put message in buffer. Message needs to start from TCP_APP_OFFSET
 	 * because TCP/IP stack will put headers in front of the message to
 	 * avoid data copying
 	 */
	/* First check if we can send data */
	if(tcp_checksend(tcp_server_socket) < 0 )
	{
		/* Not yet */
		/* Drop out and try again later */
		return 0;
	}
	else
	{
		pbuf = net_buf + TCP_APP_OFFSET;
		tmp = eth_info.total_rx_packets;

		// First signal that this is going to be a status packet
		*pbuf++ = 2;

		// Send rx packet count LSB First
		*pbuf++ = tmp & 0xFF;
		tmp >>= 8;
		*pbuf++ = tmp & 0xFF;
		tmp >>= 8;
		*pbuf++ = tmp & 0xFF;
		tmp >>= 8;
		*pbuf++ = tmp;
		// Send tx packet count LSB First
		tmp = eth_info.total_tx_packets;
		*pbuf++ = tmp & 0xFF;
		tmp >>= 8;
		*pbuf++ = tmp & 0xFF;
		tmp >>= 8;
		*pbuf++ = tmp & 0xFF;
		tmp >>= 8;
		*pbuf++ = tmp;

		// Send total running time count LSB First
		tmp = total_seconds;
		*pbuf++ = tmp & 0xFF;
		tmp >>= 8;
		*pbuf++ = tmp & 0xFF;
		tmp >>= 8;
		*pbuf++ = tmp & 0xFF;
		tmp >>= 8;
		*pbuf++ = tmp;

		/* send data */
		tcp_send(	tcp_server_socket,			// handle of TCP socket to be used for sending data
					&net_buf[TCP_APP_OFFSET],	// pointer to data buffer (start of user data)
					NETWORK_TX_BUFFER_SIZE - TCP_APP_OFFSET,	// buffer length in bytes (without space reserved at the beginning of buffer for headers)
					13);				// length of user data to be sent (in bytes)
		return 1;		// send back successful
	}
}




